@@ -1,12 +1,14 @@
 #![feature(proc_macro_diagnostic)]
 use proc_macro::TokenStream;
+use proc_macro2::Span;
 use quote::quote;
 use syn::fold::Fold;
 use syn::parse::{Parse, ParseStream, Result};
+use syn::punctuated::Punctuated;
 use syn::spanned::Spanned;
 use syn::{
     parse_macro_input, Attribute, Expr, ExprCall, ExprMethodCall, ExprPath, FnArg, Ident, ItemFn,
-    Pat, Path, PathArguments, PathSegment, Signature, Type,
+    Meta, NestedMeta, Pat, Path, PathArguments, PathSegment, Signature, Token, Type,
 };
 
 /// Right now macro should be used like:
@@ -97,8 +99,29 @@ impl Parse for Args {
                 decl,
                 var,
                 ty,
-                external_calls: Vec::new(),
-                external_methods: Vec::new(),
+                // Hardcoded external calls
+                external_calls: vec![
+                    // Common calls
+                    Ident::new("Ok", Span::call_site()),
+                    Ident::new("Err", Span::call_site()),
+                    Ident::new("new", Span::call_site()),
+                    // Instrumentation
+                    Ident::new("start_timer", Span::call_site()),
+                    Ident::new("stop_timer", Span::call_site()),
+                    Ident::new("push_syscall_result", Span::call_site()),
+                    Ident::new("push_hostcall_result", Span::call_site()),
+                ],
+                // Hardcoded External methods
+                external_methods: vec![
+                    // Common methods
+                    Ident::new("into", Span::call_site()),
+                    Ident::new("len", Span::call_site()),
+                    Ident::new("ok_or", Span::call_site()),
+                    Ident::new("fits_in_lin_mem", Span::call_site()),
+                    Ident::new("fits_in_lin_mem_usize", Span::call_site()),
+                    Ident::new("in_lin_mem", Span::call_site()),
+                    Ident::new("in_lin_mem_usize", Span::call_site()),
+                ],
             });
         }
 
@@ -144,18 +167,19 @@ impl Fold for Args {
     /// Rewrite method calls of the form o.method(...) => o.method(..., state)
     /// If the call is in self.external_methodss, ignore it
     fn fold_expr_method_call(&mut self, method_call: ExprMethodCall) -> ExprMethodCall {
-        println!("method call = {:?}", method_call);
+        let mut new_method_call = method_call;
+        *new_method_call.receiver = self.fold_expr(*new_method_call.receiver);
 
         // If method is external, ignore it
         for external_method in self.external_methods.iter() {
-            if method_call.method.to_string() == external_method.to_string() {
-                return method_call;
+            if new_method_call.method.to_string() == external_method.to_string() {
+                return new_method_call;
             }
         }
         // else, add ghost state
         let arg = self.var_as_expr();
-        let mut new_method_call = method_call;
         new_method_call.args.push(arg);
+
         new_method_call
     }
 
@@ -179,6 +203,41 @@ impl Fold for Args {
             };
         }
 
+        if attr.path.is_ident("external_calls") {
+            let func_names: Result<Meta> = attr.parse_meta();
+            if let Ok(Meta::List(l)) = func_names {
+                // iterate over punctuated list of elements
+                for meta in l.nested.iter() {
+                    if let NestedMeta::Meta(m2) = meta {
+                        let name = path_to_last_ident(m2.path().clone()).unwrap();
+                        self.external_calls.push(name.clone());
+                    }
+                }
+            }
+        }
+
+        if attr.path.is_ident("external_methods") {
+            let func_names: Result<Meta> = attr.parse_meta();
+            if let Ok(Meta::List(l)) = func_names {
+                // iterate over punctuated list of elements
+                for meta in l.nested.iter() {
+                    if let NestedMeta::Meta(m2) = meta {
+                        let name = path_to_last_ident(m2.path().clone()).unwrap();
+                        self.external_methods.push(name.clone());
+                    }
+                }
+            }
+        }
+        // match func_names {
+        //     Ok(names) => {
+        //         for name in names.iter(){
+        //             self.external_calls.push(name.clone())
+        //         }
+        //     },
+        //     Err(_) => return attr,
+        // };
+        //}
+
         attr
     }
 }
@@ -214,6 +273,42 @@ pub fn external_method(_args: TokenStream, input: TokenStream) -> TokenStream {
     input
 }
 
+#[proc_macro_attribute]
+pub fn external_calls(_args: TokenStream, input: TokenStream) -> TokenStream {
+    input
+}
+
+#[proc_macro_attribute]
+pub fn external_methods(_args: TokenStream, input: TokenStream) -> TokenStream {
+    input
+}
+
+#[proc_macro_attribute]
+pub fn dummy_macro(args: TokenStream, input: TokenStream) -> TokenStream {
+    println!("args: \"{}\"", args.to_string());
+    println!("input: \"{}\"", input.to_string());
+    input
+}
+
+// #[proc_macro_attribute]
+// pub fn fuzz_trusted(args: TokenStream, input: TokenStream) -> TokenStream {
+//     println!("args: \"{}\"", args.to_string());
+//     println!("input: \"{}\"", input.to_string());
+
+//     // // Parse the input tokens into a syntax tree
+//     // let input = parse_macro_input!(input as DeriveInput);
+
+//     // Build the output, possibly using quasi-quotation
+//     let expanded = quote! {
+//         input.to_string()
+//         input.to_string()
+//     };
+
+//     // Hand the output tokens back to the compiler
+//     TokenStream::from(expanded)
+//     input
+// }
+
 // #[cfg(feature = "enable")]
 // #[proc_macro_attribute]
 // pub fn all_with_extra_arg(args: TokenStream, input: TokenStream) -> TokenStream {
